import datetime
import functools
import json
import os
import typing
import warnings
import zoneinfo
from typing import Iterable

import fastmcp.tools
import httpx
import langchain_core.utils.function_calling
import mcp
import pydantic


def truncate_text(text: str, max_length: int, placeholder: str = "[...]") -> str:
    text = text[:max_length] + placeholder if len(text) > max_length else text
    return text


def json_dumps_for_ai(
    value: str | list | dict | pydantic.BaseModel,
    max_length: int | None = None,
    truncation_placeholder: str = "[CONTENT END] The rest of the content was truncated to fit into LLM context window.",
    **kwargs,
) -> str:
    if isinstance(value, str):
        return value

    if isinstance(value, pydantic.BaseModel):
        value = value.model_dump()

    value = json.dumps(value, ensure_ascii=False, **kwargs)
    if max_length:
        value = truncate_text(value, max_length, placeholder=truncation_placeholder)

    return value


def format_datetime_for_ai(
    date: datetime.datetime | str,
    timespec: str = "seconds",
    timezone: zoneinfo.ZoneInfo = zoneinfo.ZoneInfo("UTC"),
) -> str:
    date = datetime.datetime.fromisoformat(date) if isinstance(date, str) else date
    date = date.astimezone(timezone)
    date = date.isoformat(timespec=timespec, sep=" ")
    return date


def to_markdown_json(text: str) -> str:
    text = f"```json\n{text}\n```"
    return text


def format_json_for_ai(
    value: str | list | dict | pydantic.BaseModel, indent: int | None = 0
) -> str:
    text = json_dumps_for_ai(value, indent=indent)
    text = to_markdown_json(text)
    return text


def filter_dict_by_keys(dictionary: dict, keys: Iterable) -> dict:
    dictionary = {key: dictionary[key] for key in keys if key in dictionary}
    return dictionary


def recursively_collect_file_paths_in_directory(directory_path: str) -> list[str]:
    file_paths = []
    for root, _, files in os.walk(directory_path):
        for file in files:
            full_path = os.path.join(root, file)
            if os.path.isfile(full_path):
                file_paths.append(full_path)

    return file_paths


def get_return_type_annotation(function):
    annotation = function.__annotations__.get("return")
    return annotation


def to_fastmcp_tool(
    function,
    tags: set[str] | None,
    annotations: mcp.types.ToolAnnotations | None,
    max_output_length: int | None = None,
) -> fastmcp.tools.FunctionTool:
    """Adds a custom serializer to ensure consistent and token-efficient conversion of tool output to text."""
    return_type = get_return_type_annotation(function)
    if return_type is list or typing.get_origin(return_type) is list:
        warnings.warn(
            "When tool returns list, each of its elements will be serialized separately "
            "using fastmcp.tools.tool._convert_to_content into MCPContent."
        )

    tool = fastmcp.tools.FunctionTool.from_function(
        function,
        tags=tags,
        annotations=annotations,
        serializer=functools.partial(json_dumps_for_ai, max_length=max_output_length),
    )
    # Normalize tool parameters schema to match OpenAI's.
    # For example, it dereferences JSON schema $refs, which are not supported by many AI API providers.
    openai_tool_schema = langchain_core.utils.function_calling.convert_to_openai_tool(function)
    tool.parameters = openai_tool_schema["function"]["parameters"]
    return tool


def request_as_dict(*args, **kwargs) -> dict:
    response = httpx.get(*args, **kwargs)
    response.raise_for_status()
    response_dict = response.json()
    return response_dict


def to_datetime(
    date: datetime.date | datetime.timedelta | str | int | float | None = None,
) -> datetime.datetime:
    match date:
        case datetime.datetime():
            pass
        case datetime.date():
            date = datetime.datetime.combine(date, datetime.time())
        case None:
            date = datetime.datetime.now()
        case datetime.timedelta():
            date += datetime.datetime.now()
        case str():
            date = datetime.datetime.fromisoformat(date)
        case int() | float():
            date = datetime.datetime.fromtimestamp(date)
        case _:
            raise ValueError(f"Unexpected date type: {type(date)}")

    date = date if date.tzinfo else date.astimezone()
    # or date = date.astimezone(datetime.timezone.utc)
    return date


def join_url_components(*url_components: str) -> str:
    url = "/".join(str(i).strip("/") for i in url_components)
    return url


def try_format_json_with_indent(text: str, indent: int = 2) -> str:
    try:
        text = json.dumps(json.loads(text), indent=indent)
    except json.JSONDecodeError:
        pass

    return text


def raise_for_status(response: httpx.Response, max_text_length: int = 1000) -> None:
    if not response.is_error:
        return

    response_text = try_format_json_with_indent(response.text)
    request_text = try_format_json_with_indent(response.request.content.decode())
    if max_text_length:
        request_text = truncate_text(request_text, max_text_length)
        response_text = truncate_text(response_text, max_text_length)

    error_message = (
        f"Error in HTTP response while requesting {response.url}\n"
        f"Status code: {response.status_code}\n"
        f"Reason: {response.reason_phrase}\n"
        f"Request body: {request_text}\n"
        f"Response body: {response_text}\n"
    )
    error = httpx.HTTPStatusError(error_message, request=response.request, response=response)
    raise error


async def araise_for_status(response: httpx.Response, max_response_text_length: int = 1000) -> None:
    if not response.is_error:
        return

    await response.aread()
    raise_for_status(response, max_text_length=max_response_text_length)



import base64
import inspect
import zoneinfo
from typing import Annotated

import fastmcp.tools
import git
import gitlab
import mcp
import pydantic

from bir_mcp.utils import filter_dict_by_keys, format_datetime_for_ai, to_fastmcp_tool

RepoUrlType = Annotated[
    str,
    pydantic.Field(
        description=inspect.cleandoc("""
            The url of a GitLab repo that identifies a project in the following format:
            "https://{base_gitlab_url}/{project_path}/.git", for example:
            "https://gitlab.com/organization/namespace/project/.git"
        """)
    ),
]
BranchType = Annotated[
    str,
    pydantic.Field(description="The branch to fetch files from."),
]


def get_local_repo_metadata(
    repo_path: Annotated[str, pydantic.Field(description="The filesystem path to the repo.")],
) -> dict:
    """Retrieves metadata about the Git repo in the local filesystem, such as branch names and remote urls."""
    repo = git.Repo(repo_path, search_parent_directories=True)
    branch_names = [b.name for b in repo.branches]
    remotes = [{"name": r.name, "urls": list(r.urls)} for r in repo.remotes]
    metadata = {
        "remotes": remotes,
        "branch_names": branch_names,
        "active_branch_name": repo.active_branch.name,
    }
    return metadata


class GitLab:
    def __init__(
        self,
        url: str,
        private_token: str,
        timezone: str = "UTC",
        tag: str = "gitlab",
        max_output_length: int | None = None,
    ):
        self.gitlab = gitlab.Gitlab(
            url=url,
            private_token=private_token,
        )
        self.gitlab.auth()
        self.timezone = zoneinfo.ZoneInfo(timezone)
        self.tag = tag
        self.max_output_length = max_output_length

    def get_mcp_tools(self) -> list[fastmcp.tools.FunctionTool]:
        tools = [
            get_local_repo_metadata,
            self.get_project_metadata,
            self.list_all_repo_branch_files,
            self.get_file_content,
            self.search_in_repository,
            self.get_latest_pipeline_info,
        ]
        tools = [
            to_fastmcp_tool(
                tool,
                tags={self.tag},
                annotations=mcp.types.ToolAnnotations(readOnlyHint=True, destructiveHint=False),
                max_output_length=self.max_output_length,
            )
            for tool in tools
        ]
        return tools

    def build_mcp_server(self) -> fastmcp.FastMCP:
        tools = self.get_mcp_tools()
        server = fastmcp.FastMCP(
            name="Bir GitLab MCP server",
            instructions=inspect.cleandoc("""
                GitLab related tools.
            """),
            tools=tools,
        )
        return server

    def extract_project_path_from_url(self, url: str) -> str:
        url = url.removeprefix(self.gitlab.url)
        url = url.removesuffix(".git")
        url = url.strip("/")
        return url

    def get_project_from_url(self, url: str):
        project_path = self.extract_project_path_from_url(url)
        project = self.gitlab.projects.get(project_path)
        return project

    def get_project_metadata(self, repo_url: RepoUrlType) -> dict:
        """
        Retrieves metadata about a GitLab project identified by the repo url,
        such as name, description, last activity, topics (tags), etc.
        """
        project = self.get_project_from_url(repo_url)
        metadata = {
            "name_with_namespace": project.name_with_namespace,
            "topics": project.topics,
            "description": project.description,
            "last_activity_at": format_datetime_for_ai(
                project.last_activity_at, timezone=self.timezone
            ),
            "web_url": project.web_url,
        }
        return metadata

    def list_all_repo_branch_files(
        self,
        repo_url: RepoUrlType,
        branch: BranchType,
    ) -> dict:
        """Recursively lists all files and directories in the repository."""
        project = self.get_project_from_url(repo_url)
        tree = project.repository_tree(ref=branch, get_all=True, recursive=True)
        tree = {"files": [{"path": item["path"], "type": item["type"]} for item in tree]}
        return tree

    def get_file_content(
        self,
        repo_url: RepoUrlType,
        branch: BranchType,
        file_path: Annotated[
            str,
            pydantic.Field(
                description="The path to the file relative to the root of the repository."
            ),
        ],
    ) -> str:
        """Retrieves the text content of a specific file."""
        project = self.get_project_from_url(repo_url)
        file = project.files.get(file_path=file_path, ref=branch)
        content = base64.b64decode(file.content).decode()
        return content

    def search_in_repository(
        self,
        repo_url: RepoUrlType,
        branch: BranchType,
        query: Annotated[
            str,
            pydantic.Field(description="The text query to search for."),
        ],
    ) -> dict:
        """
        Performs a basic search for a text query within the repo's files.
        Doesn't support regex, but is case-insensitive.
        Returns a list of occurences within files, with file path, starting line in the file
        and a snippet of the contextual window in which the query was found.
        For details see the [API docs](https://docs.gitlab.com/api/search/#project-search-api).
        """
        project = self.get_project_from_url(repo_url)
        results = project.search(scope="blobs", search=query, ref=branch)
        results = [
            {
                "file_path": result["path"],
                "starting_line_in_file": result["startline"],
                "snippet": result["data"],
            }
            for result in results
        ]
        results = {
            "query": query,
            "search_results": results,
        }
        return results

    def get_latest_pipeline_info(self, repo_url: RepoUrlType) -> dict:
        """Retrieves the latest pipeline info, such as url, status, duration, commit, jobs, etc."""
        project = self.get_project_from_url(repo_url)
        pipeline = project.pipelines.latest()

        commit = project.commits.get(pipeline.sha)
        commit = filter_dict_by_keys(
            commit.attributes,
            ["title", "author_name", "web_url"],
        )

        jobs = pipeline.jobs.list(all=True)
        jobs = [
            filter_dict_by_keys(
                job.attributes,
                ["name", "status", "stage", "allow_failure", "web_url"],
            )
            for job in jobs
        ]

        info = {
            "web_url": pipeline.web_url,
            "created_at": format_datetime_for_ai(pipeline.created_at, timezone=self.timezone),
            "status": pipeline.status,
            "source": pipeline.source,
            "duration_seconds": pipeline.duration,
            "queued_duration_seconds": pipeline.queued_duration,
            "commit_sha": pipeline.sha,
            "commit": commit,
            "jobs": jobs,
        }
        return info



import inspect
from typing import Annotated

import detect_secrets
import fastmcp.tools
import mcp
import pydantic

from bir_mcp.utils import to_fastmcp_tool


def get_mcp_tools(max_output_length: int | None = None) -> list[fastmcp.tools.FunctionTool]:
    tools = [find_secrets]
    tools = [
        to_fastmcp_tool(
            tool,
            tags={"local"},
            annotations=mcp.types.ToolAnnotations(readOnlyHint=True, destructiveHint=False),
            max_output_length=max_output_length,
        )
        for tool in tools
    ]
    return tools


def build_mcp_server(max_output_length: int | None = None) -> fastmcp.FastMCP:
    tools = get_mcp_tools(max_output_length=max_output_length)
    server = fastmcp.FastMCP(
        name="MCP server with local tools",
        instructions=inspect.cleandoc("""
            Contains tool to work with local files.
        """),
        tools=tools,
    )
    return server


def find_secrets(
    file_path: Annotated[
        str,
        pydantic.Field(description="The path to the file to scan for secrets."),
    ],
) -> dict:
    """
    Scans a file for secrets using the [detect-secrets](https://github.com/Yelp/detect-secrets) tool.
    """
    secrets = detect_secrets.SecretsCollection()
    with detect_secrets.settings.default_settings():
        secrets.scan_file(file_path)

    secrets = {
        file_path: {
            "secret_type": secret.type,
            "line_number": secret.line_number,
        }
        for file_path, secret in secrets.data.items()
    }
    secrets = {"detected_secrets": secrets}
    return secrets



import datetime
import enum
import inspect
import zoneinfo
from typing import Annotated

import fastmcp.resources
import fastmcp.tools
import httpx
import mcp
import pydantic

from bir_mcp.utils import araise_for_status, filter_dict_by_keys, to_fastmcp_tool


class InfluxDbQueryType(enum.StrEnum):
    flux = enum.auto()
    influxql = enum.auto()


class Grafana:
    def __init__(
        self,
        url: str,
        auth: tuple[str, str],
        http_timeout_seconds: int = 30,
        timezone: str = "UTC",
        tag: str = "grafana",
        max_output_length: int | None = None,
    ):
        self.client = httpx.AsyncClient(
            auth=auth,
            base_url=url,
            event_hooks={"response": [araise_for_status]},
            timeout=http_timeout_seconds,
        )
        self.timezone = zoneinfo.ZoneInfo(timezone)
        self.tag = tag
        self.max_output_length = max_output_length

    def get_mcp_tools(self) -> list[fastmcp.tools.FunctionTool]:
        functions = [
            self.list_all_datasources,
            self.query_influxdb_datasource,
        ]
        tools = [
            to_fastmcp_tool(
                function,
                tags={self.tag},
                annotations=mcp.types.ToolAnnotations(readOnlyHint=True, destructiveHint=False),
                max_output_length=self.max_output_length,
            )
            for function in functions
        ]
        return tools

    def get_mcp_resources(self) -> list[fastmcp.resources.Resource]:
        functions = {
            "instruction://flux": get_flux_instruction,
        }
        resources = [
            fastmcp.resources.Resource.from_function(function, uri=uri, tags={self.tag})
            for uri, function in functions.items()
        ]
        return resources

    def build_mcp_server(self) -> fastmcp.FastMCP:
        server = fastmcp.FastMCP(
            name="Bir Grafana MCP server",
            instructions=inspect.cleandoc("""
                Grafana related tools.
            """),
            tools=self.get_mcp_tools(),
        )
        for resource in self.get_mcp_resources():
            server.add_resource(resource)

        return server

    async def list_all_datasources(self) -> dict:
        """Lists all datasources in the Grafana instance."""
        response = await self.client.get("/api/datasources")
        datasources = {
            "datasources": [
                filter_dict_by_keys(d, ["uid", "name", "typeName"]) for d in response.json()
            ]
        }
        return datasources

    async def query_influxdb_datasource(
        self,
        datasource_uid: Annotated[
            str,
            pydantic.Field(description="The uid of an InfluxDB Grafana datasource."),
        ],
        query: Annotated[
            str,
            pydantic.Field(description="The query to execute."),
        ],
        start_time: Annotated[
            str,
            pydantic.Field(
                description=inspect.cleandoc("""
                    The start time for the query in relative Grafana time units like "now", "now-5m", 
                    "now-1h", "now-7d", or in ISO 8601 format "YYYY-MM-DDTHH:MM:SS+HH:MM".
                    Can be refered to as "v.timeRangeStart" in queries.
                    Should not be older than one month for query efficiency and because data is probably 
                    not retained longer than that anyway.
                """)
            ),
        ] = "now-1h",
        end_time: Annotated[
            str,
            pydantic.Field(
                description=inspect.cleandoc("""
                    The end time for the query, in same format as start_time. 
                    Can be refered to as "v.timeRangeStop" in queries.
                """)
            ),
        ] = "now",
        query_language: Annotated[
            InfluxDbQueryType,
            pydantic.Field(
                description=inspect.cleandoc(f"""
                The language of query to execute, {InfluxDbQueryType.influxql} for SQL-like Influx query language, 
                and {InfluxDbQueryType.flux} for Flux functional data scripting language.
            """)
            ),
        ] = InfluxDbQueryType.flux,
    ) -> dict:
        """
        Executes a query against an InfluxDB datasource in the Grafana instance.
        Note that InfluxQL may not be supported by the InfluxDB datasource.
        If query returns too much data, it will lead to HTTP timeouts.
        To avoid this, the start to end time range should be as small as possible,
        and the query should aggregate data by time windows.
        Refer to Flux instruction in the MCP resources for details about the Flux language, and to
        [Grafana endpoint docs](https://grafana.com/docs/grafana/latest/developers/http_api/data_source/#query-a-data-source)
        for details about the used endpoint.
        """
        ref_id = "A"
        query = {
            "datasource": {"uid": datasource_uid},
            "refId": ref_id,
            "query": query,
            "queryType": query_language.value,
            "format": "table",
        }
        payload = {
            "queries": [query],
            "from": to_grafana_time_format(start_time),
            "to": to_grafana_time_format(end_time),
        }
        response = await self.client.post("/api/ds/query", json=payload)
        tables = []
        for frame in response.json()["results"][ref_id]["frames"]:
            table = filter_dict_by_keys(frame["schema"], ["name", "fields"])
            table["data"] = frame["data"]["values"]
            tables.append(table)

        tables = {"tables": tables}
        return tables


def to_grafana_time_format(date: str) -> str | int:
    try:
        date = datetime.datetime.fromisoformat(date)
    except ValueError:
        pass
    else:
        date = int(date.timestamp() * 1000)

    return date


def get_flux_instruction() -> str:
    """Provides information about InfluxDB Flux language, such as its data model, syntax, and usage examples."""
    instruction = """
    # Flux 
    Flux is a functional data scripting language designed for querying, analyzing, and acting on time-series data in InfluxDB.

    ## Data model
    Flux organizes data into independent buckets, similar to logical databases in SQL.
    Each bucket contains time series data. A single point is composed of:
    - measurement: The name of the thing you are measuring (e.g., "cpu", "http_requests"), similar to an SQL table or Prometheus metric name.
    - tags: Key-value pairs of metadata that describe the data (e.g., `host="server1"`, `region="us-west"`), similar to Prometheus labels.
    - fields: The actual data values (e.g., `usage_percent=99.5`, `request_count=124`), similar to Prometheus metric value.
    - timestamp: The time of the data point.

    ## Syntax
    Flux queries consist of sequentially executed pipeline of functions, joined by the pipe-forward operator "|>".
    It takes the output of one function and sends it as the input to the next function.

    ### Main Flux functions
    - from() - Specifies the InfluxDB bucket to retrieve data from. It's the starting point for query that fetches time-series data.
    - range() - Filters data based on a time range. Mandatory for time-series queries.
    - filter() - Filters rows based on column values (measurement, field, tags).
    - pivot() - Rotates data from a tall format (one row per timestamp/metric) to a wide format.
    - keep() / drop() - Filters data by column names, allowing you to keep or discard specific columns.
    - limit() - Restricts the number of rows returned.
    - group() - Groups rows together based on common column values for aggregation.
    - aggregateWindow() - Segments data into time windows and applies an aggregate function (mean, sum, etc.).
    - map() - Applies a custom function to each row to modify or add columns.
    - mean(), sum(), count(), last() - Common aggregate functions, often used inside aggregateWindow.
    - highestAverage(), highestMax(), lowestAverage(), top() - Efficient "top-n" results from a group, are significantly more performant for datasets with high tag cardinality
    - yield() - Specifies a result set to be delivered from the query.

    ### Meta functions
    - buckets() - Returns a list of all available buckets.
    - schema.measurements() - Returns a list of all measurements within a bucket.
    - schema.tagKeys() / schema.tagValues() - Returns a list of tag keys or tag values for a given measurement.

    ## Usage
    When using Flux through the Grafana's `/api/ds/query` endpoint, several variables are injected into the query,
    the most important are the `v.timeRangeStart` and `v.timeRangeStop`, which correspond to the `start_time` and `end_time` parameters
    passed to the `/api/ds/query` endpoint. So each query that fetches data from time series should look like this:

    ```flux
    // Specify the bucket name here
    from(bucket: "{bucket}")
        |> range(start: v.timeRangeStart, stop: v.timeRangeStop)
        // Filters, transformations, aggegations, etc.
        |> yield(name: "value")
    ```

    ### Examples

    #### Get all buckets
    ```flux
    buckets()
    ```

    #### Get all measurements in a given bucket
    ```flux
    // Import the schema package to access its functions
    import "influxdata/influxdb/schema"
    // Call the measurements function, specifying which bucket to look in
    schema.measurements(bucket: "{bucket}")
    ```

    #### Get all field keys for a given measurement
    ```flux
    import "influxdata/influxdb/schema"
    schema.measurementFieldKeys(
        bucket: "{bucket}",
        measurement: "{measurement}"
    )
    ```

    #### Get all tag keys for a given measurement
    ```flux
    import "influxdata/influxdb/schema"
    schema.measurementTagKeys(
        bucket: "{bucket}",
        measurement: "{measurement}"
    )
    ```

    #### Get all tag values for a given tag key
    ```flux
    import "influxdata/influxdb/schema"
    schema.measurementTagValues(
        bucket: "{bucket}",
        measurement: "{measurement}",
        tag: "{tag}"
    )
    ```

    #### Filtering and aggregation
    ```flux
    from(bucket: "{bucket}")
        |> range(start: v.timeRangeStart, stop: v.timeRangeStop)
        |> filter(fn: (r) => r._measurement == "{measurement}" and r._field == "{field}")
        // Pivot makes each unique field a new column, with timestamps as rows.
        // This is useful for creating tables or graphs with multiple series.
        |> pivot(rowKey:["_time"], columnKey: ["_field"], valueColumn: "_value")
        // Group data into 10-minute windows and calculate the mean for each window
        |> aggregateWindow(every: 10m, fn: mean)
        |> yield(name: "mean_{field}_10m")
    ```

    #### Finding lowest average values among groups
    ```flux
    from(bucket: "{bucket}")
        |> range(start: v.timeRangeStart, stop: v.timeRangeStop)
        |> filter(fn: (r) => r._measurement == "{measurement}" and r._field == "{field}")
        |> lowestAverage(n: {n}, groupColumns: ["{tag}"])
    ```
    """
    return instruction


# def build_prometheus_from_grafana_datasource_proxy(
#     grafana_url: str, datasource: str, auth: tuple[str, str], **kwargs
# ):
#     """https://grafana.com/docs/grafana/v11.2/developers/http_api/data_source/#data-source-proxy-calls-by-id"""
#     data_sources = fetch_grafana_datasources(grafana_url, auth)
#     data_source = data_sources.filter(pl.col("name") == datasource)
#     if len(data_source) != 1:
#         raise ValueError(f"Couldn't find a singular id for data source {data_source}.")

#     data_source_id = data_source["id"].item()
#     prometheus_url = join_url_components(grafana_url, "api", "datasources", "proxy", data_source_id)
#     prometheus = Prometheus(prometheus_url, auth=auth, **kwargs)
#     return prometheus


# class Prometheus:
#     """https://prometheus.io/docs/prometheus/latest/querying/api"""

#     def __init__(
#         self,
#         url: str,
#         api_version: str = "v1",
#         default_start=datetime.timedelta(days=-1),
#         default_end=None,
#         time_zone: str = "Asia/Baku",
#         auth=None,
#     ):
#         self.url = url
#         self.api_version = api_version
#         self.default_date_keys = {
#             "start": default_start,
#             "end": default_end,
#             "time": default_end,
#         }
#         self.time_zone = time_zone
#         self.auth = auth

#     def request_as_dict(self, *url_components, **params) -> dict:
#         params = {k: v for k, v in params.items() if v}
#         match url_components:
#             case (
#                 ["query"]
#                 | ["query_range"]
#                 | ["series"]
#                 | ["labels"]
#                 | ["label", _, "values"]
#                 | ["query_exemplars"]
#             ):
#                 for key, default in self.default_date_keys.items():
#                     value = params.get(key, default)
#                     params[key] = to_datetime(value).isoformat()

#         url = join_url_components(self.url, "api", self.api_version, *url_components)
#         response_dict = request_as_dict(url, params=params, auth=self.auth)
#         if response_dict["status"] == "error":
#             raise RuntimeError(
#                 f'Prometheus API request for "{join_url_components(*url_components)}" endpoint '
#                 f"with parameters {params} returned an error:\n"
#                 f"{response_dict}"
#             )

#         data = response_dict["data"]
#         return data

#     def response_metric_data_to_dataframe(self, data: dict) -> pl.DataFrame | None:
#         if not (result := data["result"]):
#             return

#         rows = []
#         for series in result:
#             match data["resultType"]:
#                 case "matrix":
#                     values = series["values"]
#                 case "vector":
#                     values = [series["value"]]

#             for timestamp, value in values:
#                 row = series["metric"] | {"timestamp": timestamp, "value": value}
#                 rows.append(row)

#         if not rows:
#             return

#         df = pl.DataFrame(rows)
#         df = df.cast({"value": pl.Float64})
#         df = df.with_columns(pl.from_epoch("timestamp").dt.convert_time_zone(self.time_zone))
#         return df

#     def query(self, query, time=None, timeout: str | None = None) -> pl.DataFrame | None:
#         """https://prometheus.io/docs/prometheus/latest/querying/api/#instant-queries"""
#         query = re.sub(r"\s+", " ", query)
#         data = self.request_as_dict("query", query=query, time=time, timeout=timeout)
#         df = self.response_metric_data_to_dataframe(data)
#         return df

#     def query_range(
#         self,
#         query: str,
#         start=None,
#         end=None,
#         step: str | float = "1m",
#         timeout: str | None = None,
#     ) -> pl.DataFrame | None:
#         """https://prometheus.io/docs/prometheus/latest/querying/api/#range-queries"""
#         data = self.request_as_dict(
#             "query_range",
#             query=re.sub(r"\s+", " ", query),
#             start=start,
#             end=end,
#             step=step,
#             timeout=timeout,
#         )
#         df = self.response_metric_data_to_dataframe(data)
#         return df

#     def get_all_metric_names(self, **kwargs) -> list[str]:
#         """https://prometheus.io/docs/prometheus/latest/querying/api/#querying-label-values"""
#         data = self.request_as_dict("label", "__name__", "values", **kwargs)
#         return data

#     def get_time_series(self, series_selector: str, start=None, end=None) -> pl.DataFrame | None:
#         """https://prometheus.io/docs/prometheus/latest/querying/api/#finding-series-by-label-matchers"""
#         data = self.request_as_dict(
#             "series", **{"match[]": series_selector, "start": start, "end": end}
#         )
#         if not data:
#             return

#         df = pl.DataFrame(data)
#         return df



import inspect
import warnings

import fastmcp
import typer

import bir_mcp.git_lab
import bir_mcp.grafana
import bir_mcp.local

# class McpCategory(enum.StrEnum):
#     gitlab = enum.auto()
#     grafana = enum.auto()
#     jira = enum.auto()
#     local = enum.auto()


def main(
    gitlab_private_token: str | None = None,
    grafana_username: str | None = None,
    grafana_password: str | None = None,
    jira_api_key: str | None = None,
    gitlab_url: str = "https://gitlab.kapitalbank.az",
    grafana_url: str = "https://yuno.kapitalbank.az",
    timezone: str = "Asia/Baku",
    max_output_length: int | None = None,
):
    server = fastmcp.FastMCP(
        name="Bir MCP server",
        instructions=inspect.cleandoc("""
            MCP server for BirBank.
        """),
    )
    server.mount(bir_mcp.local.build_mcp_server())
    if gitlab_private_token:
        subserver = bir_mcp.git_lab.GitLab(
            url=gitlab_url,
            private_token=gitlab_private_token,
            timezone=timezone,
            max_output_length=max_output_length,
        )
        server.mount(subserver.build_mcp_server(), prefix=subserver.tag)
    else:
        warnings.warn(
            "Since GitLab private token is not provided, the GitLab tools will not be available."
        )

    if grafana_username and grafana_password:
        subserver = bir_mcp.grafana.Grafana(
            url=grafana_url,
            auth=(grafana_username, grafana_password),
            timezone=timezone,
            max_output_length=max_output_length,
        )
        server.mount(subserver.build_mcp_server(), prefix=subserver.tag)
    else:
        warnings.warn(
            "Since Grafana username and password are not provided, the Grafana tools will not be available."
        )

    server.run()


if __name__ == "__main__":
    typer.run(main)
